#version 330 core

layout(location = 0) in vec3 aPos; // Position
layout(location = 2) in vec3 aNormal; // Normal
layout(location = 1) in vec2 aTexCoords; // Texture Coordinates

out vec3 FragPosView;  // View-space position
out vec3 NormalView;   // View-space normal
out vec2 TexCoords; // Texture Coordinates

uniform mat4 model; // Model matrix
uniform mat4 view; // View Matrix
uniform mat4 projection; // Projection Matrix

void main() {
    vec4 fragPosWorld = model * vec4(aPos, 1.0); // Apply model matrix
    FragPosView = vec3(view * fragPosWorld); // Convert position to view space
    NormalView = mat3(transpose(inverse(view * model))) * aNormal; // Convert normal to view space
    TexCoords = aTexCoords; // Propagate texture coordinates
    gl_Position = projection * vec4(FragPosView, 1.0); // Project view space position
}
