#version 430

layout (local_size_x = 256) in;

layout(std430, binding = 0) buffer Pos1 { vec4 Position_In[]; };
layout(std430, binding = 1) buffer Vel1 { vec4 Velocity_In[]; };
layout(std430, binding = 2) buffer Pos2 { vec4 Position_Out[]; };
layout(std430, binding = 3) buffer Vel2 { vec4 Velocity_Out[]; };
layout(binding = 4, offset = 0) uniform atomic_uint Count;

uniform float DeltaT;
uniform uint MaximumCount;
uniform uint SpawnCount;
uniform vec3 NewParticlePosition; 


const vec3 Gravity = vec3(0.0,-1.0,0.0);

// Funktion zur Erzeugung eines zuflligen Wertes basierend auf einem Seed
float rand(vec2 co) {
    return fract(sin(dot(co.xy, vec2(12.9898, 78.233))) * 43758.5453);
}

// Funktion zur Erzeugung einer zuflligen Geschwindigkeit
vec3 randomVelocity(vec3 particlePosition, float time) {
    vec2 seed1 = vec2(particlePosition.xy + time);
    vec2 seed2 = vec2(particlePosition.yz + time);
    vec2 seed3 = vec2(particlePosition.zx + time);

    float theta = rand(seed1) * 2.0 * 3.14159265;
    float phi = acos(2.0 * rand(seed2) - 1.0);

    float x = sin(phi) * cos(theta);
    float y = sin(phi) * sin(theta);
    float z = cos(phi);

    float speed = rand(seed3) * 10.0; // Multiplikator zur Steuerung der Geschwindigkeitsrange

    return vec3(x, y, z) * speed;
}

void addParticleToOutputList(vec3 pos, vec3 vel, float TTL) {
    uint nr = atomicCounterIncrement(Count);
    if (nr >= MaximumCount) {
        atomicCounterDecrement(Count);
        return;
    }
    Position_Out[nr] = vec4(pos, TTL);
    Velocity_Out[nr] = vec4(vel, 0);
}

void main() {
    uint idx = gl_GlobalInvocationID.x;

    // Spawn new particles
    if (idx < SpawnCount) {
        float TTL = 2.0f; // Time-to-live for new particles
        vec3 pos = NewParticlePosition; // Verwende die neue Uniform-Position
        float time = float(idx) + DeltaT; // Beispiel fr die Zeit oder den Framezhler
        vec3 vel = randomVelocity(pos, time)*vec3(0.1,0.02,0.1); // Zufllige Geschwindigkeit fr das Partikel
        addParticleToOutputList(pos, vel, TTL);
    }

    // Process existing particles
    if (idx >= SpawnCount && idx < MaximumCount) {
        vec3 pos = Position_In[idx - SpawnCount].xyz;
        vec3 vel = Velocity_In[idx - SpawnCount].xyz + DeltaT * Gravity;
        float TTL = Position_In[idx - SpawnCount].w - DeltaT;

        if (TTL > 0) {
            addParticleToOutputList(pos + DeltaT * vel, vel, TTL);
        }
    }
}
