#version 460

layout(points) in; // Input points
layout(triangle_strip, max_vertices = 4) out; // Output quads

uniform mat4 ProjectionMatrix;  // Projection matrix
uniform float ParticleSize;     // Particle size

in vData {  
    float TTL1; 
    vec4 viewSpacePos; 
} vertex[];

out vec2 TexCoord0;
flat out float TTL0; // TTL flat value, no interpolation

void main() {
    const vec2 size = vec2(ParticleSize);
    vec4 P = vertex[0].viewSpacePos; // Use view space position

    vec2 offsets[4] = vec2[](vec2(-0.5, -0.5), vec2(0.5, -0.5), vec2(-0.5, 0.5), vec2(0.5, 0.5));
    vec2 texCoords[4] = vec2[](vec2(0.0, 0.0), vec2(1.0, 0.0), vec2(0.0, 1.0), vec2(1.0, 1.0));

    for (int i = 0; i < 4; ++i) {
        vec4 offsetPos = vec4(offsets[i] * size, 0.0, 0.0);
        gl_Position = ProjectionMatrix * (P + offsetPos); // Transform to clip space
        TexCoord0 = texCoords[i];
        TTL0 = vertex[0].TTL1;
        EmitVertex();
    }
    EndPrimitive();
}
